@@ -512,6 +512,14 @@ class Tapis(object):
             client_key = self.client_key
         else:
             client_key = kwargs['client_key']
+        # if the tapis object does not have a username/password, look for a refresh_token and
+        # call refresh_tokens()
+        if not username or not password:
+            if self.refresh_token:
+                return self.refresh_tokens(tenant_id=self.tenant_id)
+            else:
+                raise errors.InvalidInputError('Either a refresh_token or a username and password are required '
+                                               'for get_tokens().')
         if client_id and client_key:
             auth_header = {'Authorization': get_basic_auth_header(client_id, client_key)}
         else:
@@ -664,8 +672,21 @@ class Tapis(object):
         if not self.client_key:
             raise errors.TapyClientConfigurationError(msg="client_key not configure.")
         auth_header = {'Authorization': get_basic_auth_header(self.client_id, self.client_key)}
+
+        # the refresh token could have been passed into the constructor as a raw string, in which case
+        # we need to pass it as is; otherwise, the refresh_token is likely a
+        refresh_token = None
+        if type(self.refresh_token) == str:
+            refresh_token = self.refresh_token
+        else:
+            try:
+                refresh_token = self.refresh_token.refresh_token
+            except AttributeError:
+                raise errors.InvalidInputError("The refresh_token must either be a string or a refresh_token"
+                                               "object generated by tapipy.")
+
         tokens = self.authenticator.create_token(grant_type='refresh_token',
-                                                 refresh_token=self.refresh_token.refresh_token,
+                                                 refresh_token=refresh_token,
                                                  headers=auth_header)
         self.set_access_token(tokens.access_token)
         self.set_refresh_token(tokens.refresh_token)
@@ -714,6 +735,8 @@ class Tapis(object):
         """
         if hasattr(self, 'access_token') and hasattr(self.access_token, 'access_token'):
             return self.access_token.access_token
+        if hasattr(self, 'access_token') and type(self.access_token) == str:
+            return self.access_token
         if hasattr(self, 'jwt'):
             return self.jwt
         return None
@@ -1035,6 +1058,11 @@ class Operation(object):
 
         # determine base_url --
         base_url = None
+
+        # tenant_id for the request; mostly just important for service requests; for user requests,
+        # the base_url is all that is needed.
+        tenant_id = None
+
         # if this call is coming from another tapis service,
         # we must determine base_url for a service to service request.
         if self.tapis_client.is_tapis_service:
